package net.wanji.opt.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.StrategyAndMetricsEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.databus.vo.CrossIdAndNameVO;
import net.wanji.opt.bo.BottomCurveBO;
import net.wanji.opt.bo.BottomMenuBO;
import net.wanji.opt.bo.MainlineSchemeAnalysisBO;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.dao.mapper.trend.GreenwaveInfoMapper;
import net.wanji.opt.po.strategy.SceneStrategyPO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.service.MainlineEvaluateService;
import net.wanji.opt.vo.MainlineEvaluateBottomCurveVO;
import net.wanji.opt.vo.MainlineListVO;
import net.wanji.opt.vo.MainlineSchemeAnalysisVO;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/9/19 15:24
 */
@Slf4j
@Service
public class MainlineEvaluateServiceImpl implements MainlineEvaluateService {
    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final BaseCrossTurnInfoMapper baseCrossTurnInfoMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final CrossDataHistMapper crossDataHistMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final GreenwaveInfoMapper greenwaveInfoMapper;
    private final GreenwaveHistMapper greenwaveHistMapper;
    private final GreenwaveCrossMapper greenwaveCrossMapper;
    private final BaseCrossInfoMapper baseCrossInfoMapper;
    private final GreenwaveSceneMapper greenwaveSceneMapper;
    private final SceneStrategyMapper sceneStrategyMapper;
    private final StrategyMapper strategyMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final RidInfoMapper ridInfoMapper;

    SimpleDateFormat hourMinuteFormat = new SimpleDateFormat("HH:mm");
    SimpleDateFormat dateHourMinuteFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public MainlineEvaluateServiceImpl(@Qualifier("baseCrossDirInfoMapper") BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                                       @Qualifier("baseCrossTurnInfoMapper") BaseCrossTurnInfoMapper baseCrossTurnInfoMapper,
                                       CrossBaseLaneInfoMapper crossBaseLaneInfoMapper,
                                       @Qualifier("laneInfoMapper") LaneInfoMapper laneInfoMapper,
                                       CrossDirDataHistMapper crossDirDataHistMapper, CrossDataHistMapper crossDataHistMapper, CrossTurnDataHistMapper crossTurnDataHistMapper, CrossLaneDataHistMapper crossLaneDataHistMapper, @Qualifier("greenwaveInfoMapper") GreenwaveInfoMapper greenwaveInfoMapper, @Qualifier("greenwaveHistMapper") GreenwaveHistMapper greenwaveHistMapper, @Qualifier("greenwaveCrossMapper") GreenwaveCrossMapper greenwaveCrossMapper, @Qualifier("baseCrossInfoMapper") BaseCrossInfoMapper baseCrossInfoMapper, @Qualifier("greenwaveSceneMapper") GreenwaveSceneMapper greenwaveSceneMapper, @Qualifier("sceneStrategyMapper") SceneStrategyMapper sceneStrategyMapper, @Qualifier("strategyMapper") StrategyMapper strategyMapper, @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper, @Qualifier("ridInfoMapper") RidInfoMapper ridInfoMapper) {
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.baseCrossTurnInfoMapper = baseCrossTurnInfoMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.crossDataHistMapper = crossDataHistMapper;
        this.crossTurnDataHistMapper = crossTurnDataHistMapper;
        this.crossLaneDataHistMapper = crossLaneDataHistMapper;
        this.greenwaveInfoMapper = greenwaveInfoMapper;
        this.greenwaveHistMapper = greenwaveHistMapper;
        this.greenwaveCrossMapper = greenwaveCrossMapper;
        this.baseCrossInfoMapper = baseCrossInfoMapper;
        this.greenwaveSceneMapper = greenwaveSceneMapper;
        this.sceneStrategyMapper = sceneStrategyMapper;
        this.strategyMapper = strategyMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.ridInfoMapper = ridInfoMapper;
    }

    @Override
    public List<String> bottomMenu(BottomMenuBO bo) {
        String crossId = bo.getCrossId();
        Integer scope = bo.getScope();

        if (scope == 1) {
            return buildDirMenu(crossId);
        } else if (scope == 2) {
            return buildTurnMenu(crossId);
        } else if (scope == 3) {
            return buildLaneMenu(crossId);
        }

        return null;
    }

    @Override
    public List<MainlineEvaluateBottomCurveVO> bottomCurve(BottomCurveBO bo) throws ParseException {
        String crossId = bo.getCrossId();
        Integer scope = bo.getScope();
        List<String> scopeList = bo.getScopeList();
        Integer minutes = bo.getMinutes();
        String metricCode = bo.getMetricCode();
        Date startTime = bo.getStartTime();
        Date endTime = bo.getEndTime();

        if (scope != 1 && Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getCode())) {
            throw new RuntimeException("只有路口级别可选择负载均衡度");
        }

        List<String> timeList = buildTimeList(startTime, endTime, minutes);
        List<MainlineEvaluateBottomCurveVO> res = new ArrayList<>();


        int startTimeStamp = (int)(startTime.getTime() / 1000);
        int endTimeStamp = (int)(endTime.getTime() / 1000);

        if (scope == 0) { // 路口
            // 查询路口该时段内的所有数据
            List<CrossDataHistPO> crossPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                    crossId, startTimeStamp, endTimeStamp);
            for (String timeStr : timeList) {
                Date startDatetime = calcDate(timeStr, startTime);
                MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                vo.setMetricTime(timeStr);

                // 计算 endDatetime
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(startDatetime);
                calendar.add(Calendar.MINUTE, minutes);
                Date endDatetime = calendar.getTime();
                // 转换为 10 位时间戳
                int startTimeStampPart = (int)(startDatetime.getTime() / 1000);
                int endTimeStampPart = (int)(endDatetime.getTime() / 1000);
                // 使用流式编程进行筛选
                List<CrossDataHistPO> filteredList = crossPOList.stream()
                        .filter(po -> po.getBatchTime() >= startTimeStampPart && po.getBatchTime() <= endTimeStampPart)
                        .collect(Collectors.toList());
                if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getNoStopRate)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getOneStopRate)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                    double v = filteredList.stream()
                            .mapToInt(CrossDataHistPO::getDelayTime)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getQueueLength)
                            .max()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getStopTimes)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getSpeed)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getGreenLightEfficiency)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getSturation)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getLoadBalance)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                }

                res.add(vo);
            }
        } else if (scope == 1) { // 进口道
            // 查询各进口道该时段内的所有数据
            List<CrossDirDataHistPO> dirPOList = crossDirDataHistMapper.selectByCrossIdAndStartEnd(
                    crossId, startTimeStamp, endTimeStamp);

            for (String timeStr : timeList) {
                Date startDatetime = calcDate(timeStr, startTime);

                for (String dirStr : scopeList) {
                    int index = dirStr.indexOf("进口");
                    String dirName = dirStr.substring(0, index);
                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(dirStr);

                    // 计算 endDatetime
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(startDatetime);
                    calendar.add(Calendar.MINUTE, minutes);
                    Date endDatetime = calendar.getTime();
                    // 转换为 10 位时间戳
                    int startTimeStampPart = (int)(startDatetime.getTime() / 1000);
                    int endTimeStampPart = (int)(endDatetime.getTime() / 1000);
                    // 使用流式编程进行筛选
                    List<CrossDirDataHistPO> filteredList = dirPOList.stream()
                            .filter(po -> po.getDirType().equals(dirCode))
                            .filter(po -> po.getInOutType().equals(1))
                            .filter(po -> po.getBatchTime() >= startTimeStampPart
                                    && po.getBatchTime() <= endTimeStampPart)
                            .collect(Collectors.toList());
                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossDirDataHistPO::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        } else if (scope == 2) { // 方向
            // 查询各方向该时段内的所有数据
            List<CrossTurnDataHistPO> dirPOList = crossTurnDataHistMapper.selectByCrossId (
                    crossId, endTimeStamp, startTimeStamp);

            for (String timeStr : timeList) {
                Date startDatetime = calcDate(timeStr, startTime);

                for (String turnStr : scopeList) {
                    int index = turnStr.indexOf("进口");
                    String dirName = turnStr.substring(0, index);
                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);
                    String turnName = turnStr.substring(index + 2);
                    String turnCode = TurnConvertEnum.getCodeByDesc(turnName);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(turnStr);

                    // 计算 endDatetime
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(startDatetime);
                    calendar.add(Calendar.MINUTE, minutes);
                    Date endDatetime = calendar.getTime();
                    // 转换为 10 位时间戳
                    int startTimeStampPart = (int)(startDatetime.getTime() / 1000);
                    int endTimeStampPart = (int)(endDatetime.getTime() / 1000);
                    // 使用流式编程进行筛选
                    List<CrossTurnDataHistPO> filteredList = dirPOList.stream()
                            .filter(po -> po.getInDir().equals(dirCode))
                            .filter(po -> po.getTurnType().equals(turnCode))
                            .filter(po -> po.getBatchTime() >= startTimeStampPart
                                    && po.getBatchTime() <= endTimeStampPart)
                            .collect(Collectors.toList());
                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossTurnDataHistPO::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        } else if (scope == 3) { // 车道
            // 查询各车道该时段内的所有数据
            List<CrossLaneDataHistPOExt> lanePOList = crossLaneDataHistMapper.selectByCrossId(
                    crossId, endTimeStamp, startTimeStamp);

            for (String timeStr : timeList) {
                Date startDatetime = calcDate(timeStr, startTime);

                for (String laneStr : scopeList) {
                    int indexOfJinKou = laneStr.indexOf("进口");
                    int indexOfCheDao = laneStr.indexOf("车道");
                    // 截取"进口"之前的部分
                    String dirName = laneStr.substring(0, indexOfJinKou);
                    // 截取"车道"之前的数字部分
                    String laneSortStr = laneStr.substring(indexOfCheDao - 2, indexOfCheDao);

                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);
                    Integer laneSort = Integer.parseInt(laneSortStr);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(laneStr);

                    // 计算 endDatetime
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(startDatetime);
                    calendar.add(Calendar.MINUTE, minutes);
                    Date endDatetime = calendar.getTime();
                    // 转换为 10 位时间戳
                    int startTimeStampPart = (int)(startDatetime.getTime() / 1000);
                    int endTimeStampPart = (int)(endDatetime.getTime() / 1000);
                    // 筛选方向、车道序号
                    List<CrossLaneDataHistPOExt> filteredList = lanePOList.stream()
                            .filter(po -> po.getDir().equals(dirCode))
                            .filter(po -> po.getSort().equals(laneSort))
                            .filter(po -> po.getBatchTime() >= startTimeStampPart
                                    && po.getBatchTime() <= endTimeStampPart)
                            .collect(Collectors.toList());

                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossLaneDataHistPOExt::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        }


        return res;
    }

    @Override
    public List<MainlineListVO> mainlineList() {
        List<MainlineListVO> res = new ArrayList<>();

        // 按干线名称过滤数据
        List<GreenwaveInfoPO> poList = greenwaveInfoMapper.selectAll();
        List<GreenwaveInfoPO> filteredList = poList.stream()
                .collect(Collectors.groupingBy(po -> getPrefix(po.getName())))
                .values()
                .stream()
                .filter(list -> list.size() > 1)
                .map(list -> list.get(0))
                .collect(Collectors.toList());

        for (GreenwaveInfoPO greenwaveInfoPO : filteredList) {
            MainlineListVO mainlineListVO = new MainlineListVO();

            String name = greenwaveInfoPO.getName(); // 例：旅游路-转山西路至霞景路路段:双向绿波1
            String prefix = getPrefix(name);
            mainlineListVO.setMainlineName(prefix);

            Integer greenwaveId = greenwaveInfoPO.getId();
            setCrossListAndwkt(mainlineListVO, greenwaveId);
            res.add(mainlineListVO);
        }

        return res;
    }

    private void setCrossListAndwkt(MainlineListVO mainlineListVO, Integer greenwaveId) {
        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        List<CrossIdAndNameVO> crossIdAndNameVOList = new ArrayList<>();
        StringBuilder wkt = new StringBuilder();

        for (int i = 0; i < greenwaveCrossPOList.size() - 1; i++) {
            GreenwaveCrossPO preCross = greenwaveCrossPOList.get(i);
            GreenwaveCrossPO nextCross = greenwaveCrossPOList.get(i + 1);

            // 构造路口列表
            String crossId = preCross.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
            crossIdAndNameVO.setCrossId(crossId);
            crossIdAndNameVO.setCrossName(baseCrossInfoPO.getName());
            crossIdAndNameVOList.add(crossIdAndNameVO);
            if (i == greenwaveCrossPOList.size() - 2) {
                String crossId1 = nextCross.getCrossId();
                BaseCrossInfoPO baseCrossInfoPO1 = baseCrossInfoMapper.selectById(crossId1);
                CrossIdAndNameVO crossIdAndNameVO1 = new CrossIdAndNameVO();
                crossIdAndNameVO1.setCrossId(crossId1);
                crossIdAndNameVO1.setCrossName(baseCrossInfoPO1.getName());
                crossIdAndNameVOList.add(crossIdAndNameVO1);
            }

            // 构造wkt
            RidInfoEntity ridInfo = ridInfoMapper.selectByStartEnd(preCross.getCrossId(), nextCross.getCrossId());
            if (i == 0) {
                wkt.append(ridInfo.getWkt());
            } else {
                wkt.append(";").append(ridInfo.getWkt());
            }
        }

        mainlineListVO.setCrossList(crossIdAndNameVOList);
        mainlineListVO.setWkt(wkt.toString());
    }


    private String getPrefix(String name) {
        int index = name.indexOf(':');
        return index != -1 ? name.substring(0, index).trim() : name;
    }

    @Override
    public MainlineSchemeAnalysisVO mainlineSchemeAnalysis(MainlineSchemeAnalysisBO bo) {
        String mainlineName = bo.getName();
        Date poStartTime = bo.getStartTime();
        Date poEndTime = bo.getEndTime();
        String poStartTimeStr = sdf.format(poStartTime);
        String poEndTimeStr = sdf.format(poEndTime);
        // 根据时段查询所有发生的绿波
        List<GreenwaveHistPO> greenwaveHistPOList = greenwaveHistMapper
                .selectByTimeSection(poStartTimeStr, poEndTimeStr);
        // 根据干线名称筛选结果
        List<GreenwaveHistPO> filteredList = greenwaveHistPOList.stream()
                .filter(po -> {
                    String name = po.getGreenwaveName();
                    int index = name.indexOf(':');
                    if (index != -1) {
                        String leftSide = name.substring(0, index).trim();
                        return leftSide.equals(mainlineName);
                    }
                    return false;
                })
                .collect(Collectors.toList());

        MainlineSchemeAnalysisVO res = new MainlineSchemeAnalysisVO();
        res.setSceneData(calcSceneData(filteredList));
        res.setCrossData(calcCrossData(filteredList));
        res.setEvaluateData(calcEvaluateData(filteredList));
        res.setGreenwaveData(buildGreenwaveData(filteredList));

        return res;
    }

    private List<MainlineSchemeAnalysisVO.GreenwaveData> buildGreenwaveData(List<GreenwaveHistPO> filteredList) {
        return null;
    }

    private List<String> calcEvaluateData(List<GreenwaveHistPO> filteredList) {
        int qualified = 0;
        int notQualified = 0;
        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Double noparkPassRate = greenwaveHistPO.getNoparkPassRate();
            Double cordReliability = greenwaveHistPO.getCordReliability();
            Double uncoordinatePhaseQueue = greenwaveHistPO.getUncoordinatePhaseQueue();
            Double cordQueueRatio = greenwaveHistPO.getCordQueueRatio();
            Integer greenwaveId = greenwaveHistPO.getId();
            // 确定控制策略
            GreenwaveScenePO greenwaveScenePO = greenwaveSceneMapper.selectByGreenwaveId(greenwaveId);
            Integer sceneId = greenwaveScenePO.getSceneId();
            List<SceneStrategyPO> sceneStrategyPOList = sceneStrategyMapper.selectBySceneId(sceneId);
            Optional<SceneStrategyPO> maxPriorityPO = sceneStrategyPOList.stream()
                    .max(Comparator.comparingInt(SceneStrategyPO::getSceneStrategyPriority));
            SceneStrategyPO sceneStrategyPO = maxPriorityPO.get();
            Integer strategyId = sceneStrategyPO.getStrategyId();
            StrategyPO strategyPO = strategyMapper.selectById(strategyId);
            String strategyCode = strategyPO.getStrategyCode();
            
            // 确定评价
            // 获取时段
            Integer sectionId = greenwaveHistPO.getSectionId();
            CrossSectionPO crossSectionPO = baseCrossSectionMapper.selectById(sectionId);
            String startHourMinuteStr = crossSectionPO.getStartTime();
            String endHourMinuteStr = crossSectionPO.getEndTime();
            Date day = greenwaveHistPO.getGmtModified();

            Date startDate = combineDateAndTime(day, startHourMinuteStr);
            Date endDate = combineDateAndTime(day, endHourMinuteStr);

            // 获取路口
            List<BaseCrossInfoPO> crossInfoPOList = getGreenwaveCross(greenwaveId);
            // 获取路口历史数据
            List<String> crossIdList = crossInfoPOList.stream()
                    .map(BaseCrossInfoPO::getId)
                    .collect(Collectors.toList());
            int startTimeStamp = (int) (startDate.getTime() / 1000);
            int endTimeStamp = (int) (endDate.getTime() / 1000);
            List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper
                    .selectByCrossIdsAndTimestamp(crossIdList, startTimeStamp, endTimeStamp);
            // 判断时段内的饱和度
            double saturation = crossDataHistPOList.stream()
                    .mapToDouble(CrossDataHistPO::getSturation)
                    .average()
                    .orElse(0.0);

            Integer dir = greenwaveHistPO.getDir();
            if (dir == 0 || dir == 1) { // 单向绿波
                if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                    if (saturation < 0.8) {
                        if (noparkPassRate > 0.8 && cordReliability > 0.7) {
                            // 协调不停车通过率需大于80%，并且协调方案可靠性需大于70%
                            qualified++;
                        } else if (uncoordinatePhaseQueue < 0.5) {
                            // 如果以上不满足，则非协调相位二次排队需小于50%
                            qualified++;
                        } else {
                            notQualified++;
                        }
                    } else {
                        // 协调方案可靠性需大于70%
                        if (cordReliability > 0.7) {
                            qualified++;
                        } else {
                            notQualified++;
                        }
                    }
                } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                    if (cordQueueRatio < 0.6) {
                        // 协调路段排队空间占比需小于60%
                        qualified++;
                    } else if (uncoordinatePhaseQueue < 1.0) {
                        // 如果以上不满足，则非协调相位二次排队需小于100%
                        qualified ++;
                    } else {
                        notQualified++;
                    }
                }
            } else { // 双向绿波
                if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                    // 协调不停车通过率需大于50%，并且协调方案可靠性需大于70%
                    if (noparkPassRate > 0.5 && cordReliability > 0.7) {
                        qualified++;
                    } else {
                        notQualified++;
                    }
                } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                    if (cordQueueRatio < 0.6) {
                        // 协调路段排队空间占比需小于60%
                        qualified++;
                    } else if (uncoordinatePhaseQueue < 1.0) {
                        // 如果以上不满足，则非协调相位二次排队需小于100%
                        qualified ++;
                    } else {
                        notQualified++;
                    }
                }
            }
        }
        List<String> res = new ArrayList<>();
        res.add("符合控制策略方案数 " + qualified + "个");
        res.add("不符合控制策略方案数 " + notQualified + "个");
        return res;
    }

    private List<BaseCrossInfoPO> getGreenwaveCross(Integer greenwaveId) {
        List<BaseCrossInfoPO> res = new ArrayList<>();
        List<GreenwaveCrossPO> greenwaveCrosses = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        for (GreenwaveCrossPO greenwaveCross : greenwaveCrosses) {
            String crossId = greenwaveCross.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            res.add(baseCrossInfoPO);
        }
        return res;
    }

    private Date combineDateAndTime(Date day, String hourMinuteStr) {
        String[] hourMinute = hourMinuteStr.split(":");
        int hour = Integer.parseInt(hourMinute[0]);
        int minute = Integer.parseInt(hourMinute[1]);

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(day);

        calendar.set(Calendar.HOUR_OF_DAY, hour);
        calendar.set(Calendar.MINUTE, minute);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        return calendar.getTime();
    }

    private List<MainlineSchemeAnalysisVO.CrossData> calcCrossData(List<GreenwaveHistPO> filteredList) {
        List<MainlineSchemeAnalysisVO.CrossData> res = new ArrayList<>();

        // 找出绿波对应的关键路口
        Map<Integer, String> greenwaveCrossMap = new HashMap<>();
        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Integer greenwaveId = greenwaveHistPO.getId();
            GreenwaveCrossPO greenwaveCrossPO = greenwaveCrossMapper.selectByGreenwaveIdAndKeyRoute(greenwaveId);
            Integer greenId = greenwaveCrossPO.getGreenId();
            String crossId = greenwaveCrossPO.getCrossId();
            greenwaveCrossMap.put(greenId, crossId);
        }

        // 统计关键路口次数
        Map<String, Integer> crossTimesMap = new HashMap<>();
        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Integer greenwaveHistPOId = greenwaveHistPO.getId();
            String crossId = greenwaveCrossMap.get(greenwaveHistPOId);
            crossTimesMap.put(crossId, crossTimesMap.getOrDefault(crossId, 0) + 1);
        }

        for (Map.Entry<String, Integer> entry : crossTimesMap.entrySet()) {
            MainlineSchemeAnalysisVO.CrossData crossData = new MainlineSchemeAnalysisVO.CrossData();
            String crossId = entry.getKey();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            String crossName = baseCrossInfoPO.getName();
            crossData.setName(crossName);
            crossData.setTimes(entry.getValue());
            res.add(crossData);
        }

        return res;
    }

    private List<MainlineSchemeAnalysisVO.DirectionData> calcSceneData(List<GreenwaveHistPO> filteredList) {
        List<MainlineSchemeAnalysisVO.DirectionData> res = new ArrayList<>();

        HashMap<String, Integer> map = new HashMap<>();
        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Integer dir = greenwaveHistPO.getDir();
            if (dir == 0) { // 正向
                map.put("单向绿波上行", map.getOrDefault("单向绿波上行", 0) + 1);
            } else if (dir == 1) { // 反向
                map.put("单向绿波下行", map.getOrDefault("单向绿波下行", 0) + 1);
            } else { // 双向
                map.put("双向绿波", map.getOrDefault("双向绿波", 0) + 1);
            }
        }

        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            MainlineSchemeAnalysisVO.DirectionData directionData = new MainlineSchemeAnalysisVO.DirectionData();
            directionData.setName(entry.getKey());
            directionData.setTimes(entry.getValue());
            res.add(directionData);
        }

        return res;
    }

    private Date calcDate(String timeStr, Date startTime) throws ParseException {
        // 使用 Calendar 来获取年、月、日信息
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startTime);
        // 使用 SimpleDateFormat 来获取时和分
        Date time = hourMinuteFormat.parse(timeStr);
        Calendar timeCal = Calendar.getInstance();
        timeCal.setTime(time);
        // 组合两者
        calendar.set(Calendar.HOUR_OF_DAY, timeCal.get(Calendar.HOUR_OF_DAY));
        calendar.set(Calendar.MINUTE, timeCal.get(Calendar.MINUTE));
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        // 转换回 Date 对象
        Date startDatetime = calendar.getTime();
        return startDatetime;
    }

    private List<String> buildTimeList(Date startTime, Date endTime, Integer minutes) {
        // 用于存储时间区间的列表
        List<String> timeList = new ArrayList<>();

        Calendar cal = Calendar.getInstance();
        cal.setTime(startTime);

        while (cal.getTime().before(endTime) || cal.getTime().equals(endTime)) {
            String formattedTime = hourMinuteFormat.format(cal.getTime());
            timeList.add(formattedTime);
            // 增加指定分钟数
            cal.add(Calendar.MINUTE, minutes);
        }
        return timeList;
    }

    public static String formatTime(int totalMinutes) {
        int hours = totalMinutes / 60;
        int minutes = totalMinutes % 60;
        return String.format("%02d:%02d", hours, minutes);
    }

    private List<String> buildLaneMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> dirList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO dirPo : dirList) {
            Integer dirType = dirPo.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);

            List<LaneInfoPO> laneList = laneInfoMapper.selectInDirByCrossIdAndDir(crossId, dirType);
            for (LaneInfoPO lanePO : laneList) {
                Integer turn = lanePO.getTurn();
                String turnDesc = TurnConvertEnum.getDescByKey(turn);
                Integer sort = lanePO.getSort();
                res.add(dirName + "进口" + turnDesc + sort + "车道");
            }
        }
        return res;
    }

    private List<String> buildTurnMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> dirList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO dirPo : dirList) {
            Integer dirType = dirPo.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);

            List<CrossTurnInfoPO> turnList = baseCrossTurnInfoMapper.selectByCrossIdAndDir(crossId, dirType);
            for (CrossTurnInfoPO turnPO : turnList) {
                String turnType = turnPO.getTurnType();
                String turnDesc = TurnConvertEnum.getDescByCode(turnType);
                res.add(dirName + "进口" + turnDesc);
            }
        }
        return res;
    }

    private List<String> buildDirMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> filteredList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO po : filteredList) {
            Integer dirType = po.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
            res.add(dirName + "进口");
        }
        return res;
    }


}
